package de.uni_potsdam.hpi.gds.digisim.plugin;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.beans.PropertyVetoException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import de.ksquared.jds.components.Component;
import de.ksquared.jds.components.Configurable;
import de.ksquared.jds.components.Configurable.Option.OptionType;
import de.ksquared.jds.components.Sociable;
import de.ksquared.jds.contacts.Contact;
import de.ksquared.jds.contacts.ContactList;
import de.ksquared.jds.contacts.ContactList.Alignment;
import de.ksquared.jds.contacts.ContactUtilities;
import de.ksquared.jds.contacts.InputContact;
import de.ksquared.jds.contacts.OutputContact;

public class BatchDemultiplexer extends Component implements Sociable,
		Configurable {
	
	private static final long serialVersionUID = 8812270056609600043L;

	private final int polyHeight = 20;
		
	public static final ComponentAttributes componentAttributes = 
			new ComponentAttributes(
					"Bündel Demultiplexer",
					"group.selector",
					"Bündel Demultiplexer",
					"Malte Mues",
					1);
	private int controlSignals = 2;
	private int batchSize = 2;

	
	public BatchDemultiplexer() {
		setNewContacts();
	}
	
	@Override
	public void paint(Graphics graphics) {
	
		Dimension size = getSize();
		graphics.setColor(Color.BLACK);
		graphics.drawRect(5, 0, size.width-10, size.height-polyHeight-5);
		
		int[] XArray = {5, size.width-5, size.width-5-8, 5+8};
	    int[] YArray = {size.height-polyHeight-5, size.height-polyHeight-5, size.height-5, size.height-5};
	    
		graphics.drawPolygon(XArray, YArray, 4);
		
		graphics.drawString("Demux", 10, 35);
		graphics.drawString("Bündel", 10, 25);
		
	}

	@Override
	public Dimension getSize() {
		return new Dimension((65+((controlSignals-4)*3)), (polyHeight+10+(int)(Math.pow(2, controlSignals) * (10 + batchSize * 6))));


	}

	
	private ContactList<InputContact> input;
	private ContactList<InputContact> controls;
	private ContactList<OutputContact> outputs;

	@Override
	public Contact[] getContacts() {
		return ContactUtilities.concatenateContacts(
				ContactUtilities.concatenateContacts(input.toArray(),
						outputs.toArray()), controls.toArray());

	}

	
	@Override
	public void calculate() {
		int a = 0;
		int b = controlSignals-1;
		for (InputContact x : controls) {
			if (x.isCharged()) {
				a = a + (int) Math.pow(2, b);
			}
			b--;
		}
		for (OutputContact x : outputs) {
			x.setCharged(false);
		}
		Iterator<OutputContact> outputContactIt = outputs.iterator();
		Iterator<InputContact> inputContactIt = input.iterator();
		int count = 0;
		int setContact = 0;

		while (outputContactIt.hasNext() && inputContactIt.hasNext()
				&& setContact < batchSize) {
			OutputContact outputContact = outputContactIt.next();

			if (count / batchSize == a) {
				InputContact inputContact = inputContactIt.next();
				outputContact.setCharged(inputContact.isCharged());
				setContact++;
			} else {
				count++;
			}

		}
	}

	@Override
	public Option[] getOptions() {
		return new Option[] {
		new Option("numbers", "Kontrollsignale", OptionType.NUMBER, 2),
		new Option("numbers2","Bündelbreite",OptionType.NUMBER,2)

		};
	}

	
	@Override
	public Map<Option, Object> getConfiguration() {
		Option[] options = getOptions();
		Map<Option, Object> configuration = new HashMap<Option, Object>();
		configuration.put(options[0], this.controlSignals);
		configuration.put(options[1], this.batchSize);
		return configuration;
	}

	
	@Override
	public void setConfiguration(Map<Option, Object> configuration)
			throws PropertyVetoException {
		Option[] options = getOptions();
		int option0 = (Integer) configuration.get(options[0]);
		int option1 = (Integer) configuration.get(options[1]);
		
		if (option0 < 1 || option0 > 7) {
			throw new PropertyVetoException(
					"Es sind nur zwschischen 1 und 7 Kontrollsignale erlaubt, einschließlich 1 und 7", null);
		}
		if (option1 < 1 || option1 > 8) {
			throw new PropertyVetoException(
					"Die Bündelbreite muss im Bereich zwischen 1 und 8 liegen, einschließlich 1 und 8", null);
		}
		
		this.controlSignals = option0;
		this.batchSize = option1;
		
		setNewContacts();
	}

	private void setNewContacts() {
		Dimension dimension = getSize();
		//reset output
		outputs = new ContactList<OutputContact>(this, OutputContact.class);
		outputs.setContacts((int) Math.pow(2, controlSignals) * batchSize);
		int abstand = 0;
		int index = 0;
		for (OutputContact contact : outputs) {
			if (index % batchSize == 0 && index > 0) { 
				abstand = abstand + 10;
			}
			contact.setLocation(new Point((int) dimension.getWidth(), 5
					+ abstand + index * 6));
			index++;
		}

	
		// reset controls
		controls = new ContactList<InputContact>(this, InputContact.class, controlSignals);
		controls.setContactLocations( new Point(0, (int) dimension.getHeight()),
				Alignment.HORIZONTAL);
		// reset input
		input = new ContactList<InputContact>(this, InputContact.class, batchSize);
		input.setContactLocations(new Point(0, 0), Alignment.VERTICAL);
	}
		
}

